#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <pcap.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>

#include "utils.h"
#include "pcap.h"
#include "filter.h"
#include "data.h"
#include "init.h"

extern char * ConfigFile;
extern struct s_filter * p_filter;
extern struct s_data * p_data;
extern char * device;
extern char * filemask;
extern unsigned long int next_time;
extern unsigned long int time_interval;
extern int SigHup;

void ParseCmdLine(int argc, char * argv[])
{
  char ch;
  while (EOF != (ch = getopt(argc, argv, "c:")))
    switch(ch)
    {
    case 'c':
      ConfigFile = strdup(optarg);
      break;
    default:
      /* fprintf(stderr, "wrong option\n"); */
    }
}

void InitOnce()
{
  signal(SIGHUP, SigHupHandler);
  signal(SIGTERM, SigTermHandler);
  signal(SIGKILL, SigKillHandler);

  if (NULL == ConfigFile)
  {
    ConfigFile = strdup(DEFAULT_CONFIG_FILE);
  }
}

void ReInit()
{
  configure();

  if (time_interval == 0) {
    printf("No time interval specified. Using 1 day as default.\n");
    time_interval = 24*60*60;
  }
  /* Set the time of the next dump */
  next_time = time(NULL) + time_interval;

  if (NULL == filemask) {
    printf("no output file specified. Exiting.\n");
    exit;
  }

  if (NULL == device) {
    char errbuf[PCAP_ERRBUF_SIZE];
    char * tdev;

    tdev = pcap_lookupdev(errbuf);

    if (!tdev)
    {
      fprintf(stderr, "pcap_lookupdev error: %s\n", errbuf);
      exit(1);
    }

    device = strdup(tdev);
  }

  if(!openpcap(device, 1))
  {
    printf("[ipfm] Unable to open pcap descriptor on device %s\n", device);
    /* Exit(); */
    exit(-1);
  }
}

void Init()
{
  InitOnce();
  ReInit();
}

void Clean()
{
  struct s_filter * pTempRule;
  struct s_data * pTempData;

/* clean the rules (deallocate the chained list) */

  while (NULL != p_filter)
  {
    pTempRule = p_filter;
    p_filter = p_filter->next;
    if (pTempRule)
      free(pTempRule);
    else
      fprintf(stderr, "[init.c] BUG!\n");
  }

/* clean the stats (the same) */

  while (NULL != p_data)
  {
    pTempData = p_data;
    p_data = p_data->next;
    if (pTempData)
      free(pTempData);
    else
      fprintf(stderr, "[init.c] BUG 2!\n");
  }

/* close the pcap descriptor */
  closepcap();

  if (filemask)
  {
    free(filemask);
    filemask = NULL;
  }
  if (device)
  {
    free(device);
    device = NULL;
  }
}

void Exit(int errcode)
{
  Clean();

  if (ConfigFile)
    free(ConfigFile);

  exit(errcode);
}

void SigHupHandler(int a)
{
  /*
   * on flushe, on sait jamais. il faudrait faire quelque chose 
   * pour eviter d'ecraser les fichiers au moment du flush
   */

  SigHup = 1;

  printf("IPFM : Received SigHUP\n");
}

void SigTermHandler(int a)
{
  /* 
   * use SIGTERM to flush the data so that the user 
   * can do it any time he wants
   */

  char * filename;

  printf("IPFM : Received SigTERM\n");
  filename = timefile(filemask);
  data_flush(filename);
  free (filename);
}

void SigKillHandler(int a)
{
  printf("IPFM : Received SigKILL\n");
  Exit(0);
}
